#include "stm32l1xx.h"

#ifndef __M25P64_H_
#define __M25P64_H_

#define NULL ((void *) 0)

/* Flash opcodes. */
#define OPCODE_WREN             0x06    /* Write enable */
#define OPCODE_WRDI             0x04    /* Write enable */
#define OPCODE_RDSR             0x05    /* Read status register */
#define OPCODE_WRSR             0x01    /* Write status register 1 byte */
#define OPCODE_NORM_READ        0x03    /* Read data bytes (low frequency) */
#define OPCODE_FAST_READ        0x0b    /* Read data bytes (high frequency) */
#define OPCODE_PP               0x02    /* Page program (up to 256 bytes) */
#define OPCODE_CHIP_ERASE       0xc7    /* Erase whole flash chip */
#define OPCODE_SE               0xd8    /* Sector erase (usually 64KiB) */
#define OPCODE_RDID             0x9f    /* Read JEDEC ID */
#define OPCODE_READ             0x03
#define OPCODE_RES              0xAB

// driver 
// ******************************
void m25p64_driver_init(void);
void m25p64_S_H(void);
void m25p64_S_L(void);
void m25p64_H_H(void);
void m25p64_H_L(void);
void m25p64_W_H(void);
void m25p64_W_L(void);
void spi_write_then_read(uint8_t *code, uint8_t n_tx, 
												 uint8_t *val, uint8_t n_rx);
void spi_read(uint8_t *val, uint8_t n_rx);
// ******************************

void init_m25p(void);
void read_ID(uint8_t *buf);
uint8_t read_sr(void);
void write_sr(uint8_t val);
void write_enable(void);
void write_disable(void);
uint8_t wait_till_ready(void);
void erasce_chip(void);
void erase_sector(uint32_t offset);
void read_flash(uint32_t adr, uint16_t len, uint8_t *buf);
void write_flash(uint32_t adr, uint16_t len, uint8_t *buf);
uint8_t read_sign(void);

#endif
