#include "m25p64.h"
#include "wait_t6.h"

//   
// _H - High -  
// _W - High -  
// _S - Low - 
void init_m25p(void)
{
	m25p64_driver_init();
	
	m25p64_H_H();
	m25p64_W_H();
	m25p64_S_L();
}

//  ID
//   buf - 3 
void read_ID(uint8_t *buf)
{
	uint8_t code = OPCODE_RDID;
	
	m25p64_S_L();
	spi_write_then_read(&code, 1, &buf[0], 3);
	m25p64_S_H();
}

//   
uint8_t read_sr(void)
{
	uint8_t code = OPCODE_RDSR;
	uint8_t val;
	
	m25p64_S_L();
	spi_write_then_read(&code, 1, &val, 1);
	m25p64_S_H();
	
	return val;
}

//    
void write_sr(uint8_t val)
{
	uint8_t data[2];
	
	data[0] = OPCODE_WRSR;
	data[1] = val;
	
	write_enable();
	
	m25p64_S_L();
	spi_write_then_read(&data[0], 2, NULL, 0);
	m25p64_S_H();
	while(wait_till_ready() == 1);
}

//  
void write_enable(void)
{
	uint8_t code = OPCODE_WREN;
	
	m25p64_S_L();
	spi_write_then_read(&code, 1, NULL, 0);
	m25p64_S_H();
}

//  
void write_disable(void)
{
	uint8_t code = OPCODE_WRDI;
	
	m25p64_S_L();
	spi_write_then_read(&code, 1, NULL, 0);
	m25p64_S_H();
}

//   ,      (timeout)
uint8_t wait_till_ready(void)
{
	uint8_t sr;
	
	sr = read_sr();
	sr = sr & 1;
	return(sr);
}

//   flash 
void erasce_chip(void)
{
	uint8_t code = OPCODE_CHIP_ERASE;
	
	write_enable();

	m25p64_S_L();
	spi_write_then_read(&code, 1, NULL, 0);
	m25p64_S_H();
	while(wait_till_ready() == 1);
}

//   flash 
void erase_sector(uint32_t offset)
{
	uint8_t code[4];
	code[0] = OPCODE_SE;
	code[1] = offset >> 16;
	code[2] = offset >> 8;
	code[3] = offset;
	
	write_enable();
	
	m25p64_S_L();
	spi_write_then_read(&code[0], 4, NULL, 0);
	m25p64_S_H();
	while(wait_till_ready() == 1);
}

//  flash 
void read_flash(uint32_t adr, uint16_t len, uint8_t *buf)
{
	uint8_t code[4];
	
	code[0] = OPCODE_READ;
	code[1] = adr >> 16;
	code[2] = adr >> 8;
	code[3] = adr;

	m25p64_S_L();
	spi_write_then_read(&code[0], 4, buf, len);
	m25p64_S_H();
}

//  flash 
void write_flash(uint32_t adr, uint16_t len, uint8_t *buf)
{
	uint8_t code[16];
	
	code[0] = OPCODE_PP;
	code[1] = adr >> 16;
	code[2] = adr >> 8;
	code[3] = adr;
	code[4] = buf[0];
	code[5] = buf[1];
	code[6] = buf[2];
	
	write_enable();
	
	m25p64_S_L();
	spi_write_then_read(&code[0], 4, NULL, 0);
	spi_write_then_read(&buf[0], len, NULL, 0);
	m25p64_S_H();
	while(wait_till_ready() == 1);
}

uint8_t read_sign(void)
{
	// 4 .. 3 dummy cycle
	uint8_t code[4];
	uint8_t tmp;
	
	code[0] = OPCODE_RES;
	
	m25p64_S_L();
	spi_write_then_read(&code[0], 4, &tmp, 1);
	m25p64_S_H();
	
	return(tmp);
}

