#include "stm32l1xx.h"
#include "wait_t6.h"

//     spi 
void m25p64_driver_init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	SPI_InitTypeDef  SPI_InitStructure;
	
	//    
	// S = CS = PB0; W = write protect = PB1; H = hold = PA4
	// *******************************************************
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
	
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	// *******************************************************
	
	//  SPI  m25p64
	// *******************************************************
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE);
	
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF; 
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz; 
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL; 
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7; 
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP; 
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	GPIO_PinAFConfig(GPIOA, GPIO_PinSource5, GPIO_AF_SPI1);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource6, GPIO_AF_SPI1);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource7, GPIO_AF_SPI1);
	
	SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex; 
	SPI_InitStructure.SPI_Mode =  SPI_Mode_Master; 
	SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b; 
	SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low; 
	SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge; 
	SPI_InitStructure.SPI_NSS = SPI_NSS_Soft; 
	SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_128; 
	SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB; 
	SPI_Init(SPI1, &SPI_InitStructure);
	SPI_Cmd(SPI1, ENABLE);
	// *******************************************************
}

//  
//tSHSL = 100ns
void m25p64_S_H(void)
{
	GPIO_SetBits(GPIOB, GPIO_Pin_0);
	wait_us(1);
}
void m25p64_S_L(void)
{
	GPIO_ResetBits(GPIOB, GPIO_Pin_0);
}
void m25p64_H_H(void)
{
	GPIO_SetBits(GPIOA, GPIO_Pin_4);
}
void m25p64_H_L(void)
{
	GPIO_ResetBits(GPIOA, GPIO_Pin_4);
}

//tWHSL = 20ns
void m25p64_W_H(void)
{
	GPIO_SetBits(GPIOB, GPIO_Pin_1);
	wait_us(1);
}

//tSHWL = 100ns
void m25p64_W_L(void)
{
	wait_us(1);
	GPIO_ResetBits(GPIOB, GPIO_Pin_1);
}

//     spi
void spi_write_then_read(uint8_t *code, uint8_t n_tx, 
												 uint8_t *val, uint8_t n_rx)
{
	uint8_t temp;
	
	for(temp = 0; temp < (n_tx + n_rx); temp++)
	{
		while(SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE) == RESET);
		SPI_I2S_SendData (SPI1, code[temp]);
		while(SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_RXNE) == RESET);
		
		if(temp >= n_tx)		
			val[(temp - n_tx)] = SPI_I2S_ReceiveData(SPI1);
		else
			SPI_I2S_ReceiveData(SPI1);
	}
}

void spi_read(uint8_t *val, uint8_t n_rx)
{
	uint8_t temp;
	
	for(temp = 0; temp < n_rx; temp++)
	{
		while(SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_RXNE) == RESET);
		val[temp] = SPI_I2S_ReceiveData(SPI1);
	}
}
